@@ -10,15 +10,18 @@ import kubernetes
 import yaml
 from kubernetes.dynamic.exceptions import MethodNotAllowedError
 from openshift.dynamic import DynamicClient
-from openshift.dynamic.exceptions import ConflictError, NotFoundError
+from openshift.dynamic.exceptions import (
+    ConflictError,
+    InternalServerError,
+    NotFoundError,
+    ServerTimeoutError,
+)
 from openshift.dynamic.resource import ResourceField
 from packaging.version import Version
 
 from ocp_resources.constants import (
-    DEFAULT_CLUSTER_RETRY_EXCEPTIONS,
     NOT_FOUND_ERROR_EXCEPTION_DICT,
     PROTOCOL_ERROR_EXCEPTION_DICT,
-    TIMEOUT_1MINUTE,
     TIMEOUT_4MINUTES,
 )
 from ocp_resources.event import Event
@@ -30,7 +33,17 @@ from ocp_resources.utils import (
 )
 
 
-LOGGER = get_logger(__name__)
+DEFAULT_CLUSTER_RETRY_EXCEPTIONS = {
+    ConnectionAbortedError: [],
+    ConnectionResetError: [],
+    InternalServerError: [
+        "etcdserver: leader changed",
+        "etcdserver: request timed out",
+    ],
+    ServerTimeoutError: [],
+}
+
+LOGGER = get_logger(name=__name__)
 MAX_SUPPORTED_API_VERSION = "v2"
 
 
@@ -38,76 +51,6 @@ def kube_v1_api(api_client):
     return kubernetes.client.CoreV1Api(api_client=api_client)
 
 
-def _collect_instance_data(directory, resource_object):
-    with open(os.path.join(directory, f"{resource_object.name}.yaml"), "w") as fd:
-        fd.write(resource_object.instance.to_str())
-
-
-def _collect_pod_logs(dyn_client, resource_item, **kwargs):
-    return kube_v1_api(api_client=dyn_client.client).read_namespaced_pod_log(
-        name=resource_item.metadata.name,
-        namespace=resource_item.metadata.namespace,
-        **kwargs,
-    )
-
-
-def _collect_virt_launcher_data(dyn_client, directory, resource_object):
-    if resource_object.kind == "VirtualMachineInstance":
-        for pod in dyn_client.resources.get(kind="Pod").get().items:
-            pod_name = pod.metadata.name
-            pod_instance = dyn_client.resources.get(
-                api_version=pod.apiVersion, kind=pod.kind
-            ).get(name=pod_name, namespace=pod.metadata.namespace)
-            if pod_name.startswith("virt-launcher"):
-                with open(os.path.join(directory, f"{pod_name}.log"), "w") as fd:
-                    fd.write(
-                        _collect_pod_logs(
-                            dyn_client=dyn_client,
-                            resource_item=pod,
-                            container="compute",
-                        )
-                    )
-
-                with open(os.path.join(directory, f"{pod_name}.yaml"), "w") as fd:
-                    fd.write(pod_instance.to_str())
-
-
-def _collect_data_volume_data(dyn_client, directory, resource_object):
-    if resource_object.kind == "DataVolume":
-        cdi_worker_prefixes = ("importer", "cdi-upload")
-        for pod in dyn_client.resources.get(kind="Pod").get().items:
-            pod_name = pod.metadata.name
-            pod_instance = dyn_client.resources.get(
-                api_version=pod.apiVersion, kind=pod.kind
-            ).get(name=pod_name, namespace=pod.metadata.namespace)
-            if pod_name.startswith(cdi_worker_prefixes) or pod_name.endswith(
-                "source-pod"
-            ):
-                with open(os.path.join(directory, f"{pod_name}.log"), "w") as fd:
-                    fd.write(
-                        _collect_pod_logs(dyn_client=dyn_client, resource_item=pod)
-                    )
-
-                with open(os.path.join(directory, f"{pod_name}.yaml"), "w") as fd:
-                    fd.write(pod_instance.to_str())
-
-
-def _collect_data(resource_object, dyn_client=None):
-    dyn_client = (
-        dyn_client
-        if dyn_client
-        else DynamicClient(kubernetes.config.new_client_from_config())
-    )
-    directory = os.environ.get("TEST_DIR_LOG")
-    _collect_instance_data(directory=directory, resource_object=resource_object)
-    _collect_virt_launcher_data(
-        dyn_client=dyn_client, directory=directory, resource_object=resource_object
-    )
-    _collect_data_volume_data(
-        dyn_client=dyn_client, directory=directory, resource_object=resource_object
-    )
-
-
 def _find_supported_resource(dyn_client, api_group, kind):
     results = dyn_client.resources.search(group=api_group, kind=kind)
     sorted_results = sorted(
@@ -260,7 +203,6 @@ class Resource:
     api_group = None
     api_version = None
     singular_name = None
-    timeout_seconds = TIMEOUT_1MINUTE
 
     class Status:
         SUCCEEDED = "Succeeded"
@@ -314,8 +256,9 @@ class Resource:
         CONSOLE_OPENSHIFT_IO = "console.openshift.io"
         DATA_IMPORT_CRON_TEMPLATE_KUBEVIRT_IO = "dataimportcrontemplate.kubevirt.io"
         EVENTS_K8S_IO = "events.k8s.io"
-        FORKLIFT_KONVEYOR_IO = "forklift.konveyor.io"
         FLAVOR_KUBEVIRT_IO = "flavor.kubevirt.io"
+        FORKLIFT_KONVEYOR_IO = "forklift.konveyor.io"
+        INSTANCE_TYPE_KUBEVIRT_IO = "instancetype.kubevirt.io"
         HCO_KUBEVIRT_IO = "hco.kubevirt.io"
         HOSTPATHPROVISIONER_KUBEVIRT_IO = "hostpathprovisioner.kubevirt.io"
         IMAGE_OPENSHIFT_IO = "image.openshift.io"
@@ -389,25 +332,12 @@ class Resource:
         node_selector_labels=None,
         config_file=None,
         context=None,
-        timeout_seconds=TIMEOUT_1MINUTE,
     ):
         """
-        Create an API resource
+        Create a API resource
 
         Args:
             name (str): Resource name
-            client (DynamicClient): Dynamic client for connecting to a remote cluster
-            teardown (bool): Indicates if this resource would need to be deleted
-            privileged_client (DynamicClient): Instance of Dynamic client
-            yaml_file (str): yaml file for the resource
-            delete_timeout (int): timeout associated with delete action
-            dry_run (bool): dry run
-            node_selector (str): node selector
-            node_selector_labels (str): node selector labels
-            config_file (str): Path to config file for connecting to remote cluster.
-            context (str): Context name for connecting to remote cluster.
-            timeout_seconds (int): timeout for a get api call, call out be terminated after this many seconds
-
         """
         if not self.api_group and not self.api_version:
             raise NotImplementedError(
@@ -433,9 +363,6 @@ class Resource:
         self.node_selector_spec = self._prepare_node_selector_spec()
         self.res = None
         self.yaml_file_contents = None
-        self.logger = get_logger(name=f"{__name__.rsplit('.')[0]} {self.kind}")
-        self.timeout_seconds = timeout_seconds
-        self._set_client_and_api_version()
 
     def _prepare_node_selector_spec(self):
         if self.node_selector:
@@ -462,21 +389,24 @@ class Resource:
                     with open(self.yaml_file, "r") as stream:
                         self.yaml_file_contents = stream.read()
 
-            self.res = yaml.safe_load(stream=self.yaml_file_contents)
-            self.res.get("metadata", {}).pop("resourceVersion", None)
-            self.name = self.res["metadata"]["name"]
+            self.resource_dict = yaml.safe_load(stream=self.yaml_file_contents)
+            self.resource_dict.get("metadata", {}).pop("resourceVersion", None)
+            self.name = self.resource_dict["metadata"]["name"]
         else:
-            self.res = {
+            self._set_client_and_api_version()
+            self.resource_dict = {
                 "apiVersion": self.api_version,
                 "kind": self.kind,
                 "metadata": {"name": self.name},
             }
 
+        return self.resource_dict
+
     def to_dict(self):
         """
         Generate intended dict representation of the resource.
         """
-        self._base_body()
+        return self._base_body()
 
     def __enter__(self):
         signal(SIGINT, self._sigint_handler)
@@ -525,14 +455,6 @@ class Resource:
         return self
 
     def clean_up(self):
-        if os.environ.get("CNV_TEST_COLLECT_LOGS", "0") == "1":
-            try:
-                _collect_data(resource_object=self)
-            except Exception as exception_:
-                LOGGER.warning(
-                    f"Log collector failed to collect info for {self.kind} {self.name}\nexception: {exception_}"
-                )
-
         """
         For debug, export SKIP_RESOURCE_TEARDOWN to skip resource teardown.
         Spaces are important in the export dict
@@ -558,7 +480,7 @@ class Resource:
             user_exported_args=skip_resource_teardown,
             check_exists=False,
         ):
-            self.logger.warning(
+            LOGGER.warning(
                 f"Skip resource {self.kind} {self.name} teardown. Got {_export_str}={skip_resource_teardown}"
             )
             return
@@ -574,10 +496,8 @@ class Resource:
 
         get_kwargs = {"singular_name": singular_name} if singular_name else {}
         return dyn_client.resources.get(
-            kind=cls.kind,
-            api_version=cls.api_version,
-            **get_kwargs,
-        ).get(*args, **kwargs, timeout_seconds=cls.timeout_seconds)
+            kind=cls.kind, api_version=cls.api_version, **get_kwargs
+        ).get(*args, **kwargs)
 
     def _prepare_singular_name_kwargs(self, **kwargs):
         kwargs = kwargs if kwargs else {}
@@ -637,14 +557,13 @@ class Resource:
         Raises:
             TimeoutExpiredError: If resource not exists.
         """
-        self.logger.info(f"Wait until {self.kind} {self.name} is created")
+        LOGGER.info(f"Wait until {self.kind} {self.name} is created")
         samples = TimeoutSampler(
             wait_timeout=timeout,
             sleep=sleep,
             exceptions_dict={
                 **PROTOCOL_ERROR_EXCEPTION_DICT,
                 **NOT_FOUND_ERROR_EXCEPTION_DICT,
-                **DEFAULT_CLUSTER_RETRY_EXCEPTIONS,
             },
             func=lambda: self.exists,
         )
@@ -662,7 +581,7 @@ class Resource:
         Raises:
             TimeoutExpiredError: If resource still exists.
         """
-        self.logger.info(f"Wait until {self.kind} {self.name} is deleted")
+        LOGGER.info(f"Wait until {self.kind} {self.name} is deleted")
         return self.client_wait_deleted(timeout=timeout)
 
     @property
@@ -707,14 +626,11 @@ class Resource:
             TimeoutExpiredError: If resource in not in desire status.
         """
         stop_status = stop_status if stop_status else self.Status.FAILED
-        self.logger.info(f"Wait for {self.kind} {self.name} status to be {status}")
+        LOGGER.info(f"Wait for {self.kind} {self.name} status to be {status}")
         samples = TimeoutSampler(
             wait_timeout=timeout,
             sleep=sleep,
-            exceptions_dict={
-                **PROTOCOL_ERROR_EXCEPTION_DICT,
-                **DEFAULT_CLUSTER_RETRY_EXCEPTIONS,
-            },
+            exceptions_dict=PROTOCOL_ERROR_EXCEPTION_DICT,
             func=self.api.get,
             field_selector=f"metadata.name=={self.name}",
             namespace=self.namespace,
@@ -736,16 +652,15 @@ class Resource:
 
         except TimeoutExpiredError:
             if current_status:
-                self.logger.error(
-                    f"Status of {self.kind} {self.name} is {current_status}"
-                )
+                LOGGER.error(f"Status of {self.kind} {self.name} is {current_status}")
             raise
 
-    def create(self, wait=False):
+    def create(self, body=None, wait=False):
         """
         Create resource.
 
         Args:
+            body (dict): Resource data to create.
             wait (bool) : True to wait for resource status.
 
         Returns:
@@ -754,25 +669,34 @@ class Resource:
         Raises:
             ValueMismatch: When body value doesn't match class value
         """
-        if not self.res:
-            self.to_dict()
+        data = self.to_dict()
+        if body:
+            kind = body["kind"]
+            name = body.get("name")
+            api_version = body["apiVersion"]
+            if kind != self.kind:
+                raise ValueMismatch(f"{kind} != {self.kind}")
+            if name and name != self.name:
+                raise ValueMismatch(f"{name} != {self.name}")
+            if api_version != self.api_version:
+                raise ValueMismatch(f"{api_version} != {self.api_version}")
 
-        self.logger.info(f"Create {self.kind} {self.name}")
-        self.logger.info(f"Posting {self.res}")
-        self.logger.debug(f"\n{yaml.dump(self.res)}")
-        resource_ = self.api.create(
-            body=self.res, namespace=self.namespace, dry_run=self.dry_run
-        )
-        if wait and resource_:
+            data.update(body)
+
+        LOGGER.info(f"Create {self.kind} {self.name}")
+        LOGGER.info(f"Posting {data}")
+        LOGGER.debug(f"\n{yaml.dump(data)}")
+        res = self.api.create(body=data, namespace=self.namespace, dry_run=self.dry_run)
+        if wait and res:
             return self.wait()
-        return resource_
+        return res
 
     def delete(self, wait=False, timeout=TIMEOUT_4MINUTES, body=None):
-        self.logger.info(f"Delete {self.kind} {self.name}")
+        LOGGER.info(f"Delete {self.kind} {self.name}")
         if self.exists:
             data = self.instance.to_dict()
-            self.logger.info(f"Deleting {data}")
-            self.logger.debug(f"\n{yaml.dump(data)}")
+            LOGGER.info(f"Deleting {data}")
+            LOGGER.debug(f"\n{yaml.dump(data)}")
 
         try:
             res = self.api.delete(name=self.name, namespace=self.namespace, body=body)
@@ -793,7 +717,7 @@ class Resource:
         Returns:
            str: Status
         """
-        self.logger.info(f"Get {self.kind} {self.name} status")
+        LOGGER.info(f"Get {self.kind} {self.name} status")
         return self.instance.status.phase
 
     def update(self, resource_dict):
@@ -803,8 +727,8 @@ class Resource:
         Args:
             resource_dict: Resource dictionary
         """
-        self.logger.info(f"Update {self.kind} {self.name}:\n{resource_dict}")
-        self.logger.debug(f"\n{yaml.dump(resource_dict)}")
+        LOGGER.info(f"Update {self.kind} {self.name}:\n{resource_dict}")
+        LOGGER.debug(f"\n{yaml.dump(resource_dict)}")
         self.api.patch(
             body=resource_dict,
             namespace=self.namespace,
@@ -816,14 +740,15 @@ class Resource:
         Replace resource metadata.
         Use this to remove existing field. (update() will only update existing fields)
         """
-        self.logger.info(f"Replace {self.kind} {self.name}: \n{resource_dict}")
-        self.logger.debug(f"\n{yaml.dump(resource_dict)}")
+        LOGGER.info(f"Replace {self.kind} {self.name}: \n{resource_dict}")
+        LOGGER.debug(f"\n{yaml.dump(resource_dict)}")
         self.api.replace(body=resource_dict, name=self.name, namespace=self.namespace)
 
     @staticmethod
     def retry_cluster_exceptions(
         func, exceptions_dict=DEFAULT_CLUSTER_RETRY_EXCEPTIONS, **kwargs
     ):
+
         sampler = TimeoutSampler(
             wait_timeout=10,
             sleep=1,
@@ -908,7 +833,7 @@ class Resource:
         Raises:
             TimeoutExpiredError: If Pod condition in not in desire status.
         """
-        self.logger.info(
+        LOGGER.info(
             f"Wait for {self.kind}/{self.name}'s '{condition}' condition to be '{status}'"
         )
         samples = TimeoutSampler(
@@ -1137,20 +1062,22 @@ class NamespacedResource(Resource):
         return self.api.get(name=self.name, namespace=self.namespace)
 
     def _base_body(self):
-        if not self.res:
-            super(NamespacedResource, self)._base_body()
-
+        res = super(NamespacedResource, self)._base_body()
         if self.yaml_file:
-            self.namespace = self.res["metadata"].get("namespace", self.namespace)
+            self.namespace = self.resource_dict["metadata"].get(
+                "namespace", self.namespace
+            )
 
         if not self.namespace:
             raise ValueError("Namespace must be passed or specified in the YAML file.")
 
         if not self.yaml_file:
-            self.res["metadata"]["namespace"] = self.namespace
+            res["metadata"]["namespace"] = self.namespace
+
+        return res
 
     def to_dict(self):
-        self._base_body()
+        return self._base_body()
 
 
 class ResourceEditor:
